/*
 * Problem    : museum
 * Author     : Daniel Neiter
 * Language   : C++
 * Compiler   : GCC 3.4.2
 * Algorithm  : Dynamic programming 
 * Complexity : O(N^2) 
 */

#include <cstdio>
#include <iostream>

using namespace std;

const char *infile="museum.dat";
const char *outfile="museum.sol";

const int maxN=100000;

struct edge {
     int w;
     edge *next;
     edge(int w=0,edge *next=0):w(w),next(next) {}
};

edge *adj[maxN+1];
edge edgs[2*maxN];
int par[maxN+1];
int eCnt;
int n,mod,cl;
int cycle[maxN];
int lock_e1,lock_e2;
bool lock_v[maxN+1];
bool visited[maxN+1];
int dp[maxN+1][2];              // 0 - normal, 1 - hang on free edge

inline void AddEdge(int v,int w) {
     adj[v]=&(edgs[eCnt++]=edge(w,adj[v]));
     adj[w]=&(edgs[eCnt++]=edge(v,adj[w]));
}

bool find_cycle(int v,int p) {
     par[v]=p;
     for(edge *e=adj[v];e;e=e->next) 
          if(par[e->w]<0) {
               if(find_cycle(e->w,v))return true;
          }
          else if(e->w!=p) {
               for(cl=0;v!=e->w;v=par[v]) 
                    cycle[cl++]=v;
               cycle[cl++]=e->w;
               return true;
          }
     return false;
}

#define GOOD(e) ((e)->w!=par&&!lock_v[(e)->w]&&                         \
                      !(v==lock_e1&&(e)->w==lock_e2||v==lock_e2&&(e)->w==lock_e1))

void calc(int v,int par=-1) {
     int cur;
     visited[v]=true;
     dp[v][1]=1;
     for(edge *e=adj[v];e;e=e->next) 
          if(GOOD(e)) {
               calc(e->w,v);               
               dp[v][1]=(dp[v][1]*dp[e->w][0])%mod;
          }     
     dp[v][0]=dp[v][1];
     for(edge *e2=adj[v];e2;e2=e2->next) 
          if(GOOD(e2)) {
               cur=1;
               for(edge *e=adj[v];e;e=e->next) 
                    if(GOOD(e)) {
                         if(e2->w==e->w)cur=(cur*dp[e->w][1])%mod;
                         else cur=(cur*dp[e->w][0])%mod;
                    }     
               dp[v][0]=(dp[v][0]+cur)%mod;
          }
}

int calc() {
     int res=1;
     for(int i=1;i<=n;i++)
          if(!lock_v[i]&&!visited[i]) {
               calc(i);
               res=(res*dp[i][0])%mod;
          }
     return res;
}

int main() {
     int i,a,b;
     int res;
     FILE *ifs=fopen(infile,"r");
     FILE *ofs=fopen(outfile,"w");

     fscanf(ifs,"%d%d",&n,&mod);
     for(i=1;i<=n;i++) {
          fscanf(ifs,"%d%d",&a,&b);
          AddEdge(a,b);
          par[i]=-1;
     }

     find_cycle(1,0);

     res=0;
     // remove one edge from the cycle, do not use it
     lock_e1=cycle[0];lock_e2=cycle[1];
     for(i=1;i<=n;++i)lock_v[i]=visited[i]=false;
     res=(res+calc())%mod;
     // use the edge
     lock_e1=-1;lock_e2=-1;
     for(i=1;i<=n;++i)lock_v[i]=visited[i]=false;
     lock_v[cycle[0]]=lock_v[cycle[1]]=true;
     res=(res+calc())%mod;
     // use the cycle
     lock_e1=-1;lock_e2=-1;
     for(i=1;i<=n;++i)lock_v[i]=visited[i]=false;
     for(i=0;i<cl;++i)lock_v[cycle[i]]=true;
     res=(res+2*calc())%mod;     
     
     fprintf(ofs,"%d\n",res);
     
     fclose(ifs);
     fclose(ofs);
     return 0;
}
